--[[---------------------------------------------
	Project: bank_vault
	Dev: x[N]ir - (USGN #4841)
--]]---------------------------------------------

--// Checking if the mod is already launched
if bv then
	print("\169255000000[ERROR]: BANK_VAULT Mod is already launched !")
	return 0
end

--// Libs
local sqrt = math.sqrt
local random = math.random
local floor = math.floor
local match = string.match

--// bank_vault table
bv = {}					
--// array of bank vault objects				
bv.Vaults = {}
--// array of suitcases objects
bv.SuitCases = {}
--// Array which indicates whether a player is carrying a suitcase
bv.carrying = {}
--// differents states             
bv.STATE = {
	SAFE = 0,
	ROBBED = 1,
	DROPPED = 0,
	CARRIED = 1,
} 
--// Your Bank name
bv.BANK_NAME = "Bank of America"              
bv.BANK_CURRENCY = "$"
--// Robbery music
bv.ROBBERY_MUSIC = "xNir/bank_vault/music.ogg"
bv.COLLECT_SOUND = "xNir/bank_vault/collect.ogg"
--// images paths
bv.VAULT_MODEL = "gfx/xNir/bank_vault/model_bv.png"
bv.SUITCASE_MODEL = "gfx/xNir/bank_vault/model_suitcase.png"
bv.SUITCASE_DROPPED_MODEL = "gfx/xNir/bank_vault/model_suitcase_dropped.png"
--// use range in tiles
bv.VAULT_USE_RANGE = 2
bv.SUITCASE_USE_RANGE = 1
--// How far (in tiles) the player should be from the vault in order to
--// collect the money from the suitcase 
bv.SUITCASE_COLLECT_RANGE_FROM_VAULT = 10
--// vault starting money   
bv.START_MONEY = 5000    
--// Money Increase (Range of values) per second
bv.MIN_INCREASE_AMOUNT = 50
bv.MAX_INCREASE_AMOUNT = 500
--// SuitCases money range
bv.MIN_MONEY_AMOUNT = 50
bv.MAX_MONEY_AMOUNT = 500
--// Range in seconds between each robbery
bv.MIN_ROBBING_INTERVAL = 30
bv.MAX_ROBBING_INTERVAL = 60
--// Robbering duration in seconds
bv.ROBBERY_DURATION = 100
--// Delay in seconds between each collect
bv.SUITCASE_USE_DELAY = 0
--// CT Reward range
bv.MIN_REWARD = 500
bv.MAX_REWARD = 5000
--// How long can a dropped suitcase last on the ground in seconds
bv.SUITCASE_DISAPPEAR_DELAY = 120

--// Returns the distance between two objects
function bv.dist(x1, y1, x2, y2)
	return sqrt( ((y2 - y1) ^ 2) + ((x2 - x1) ^ 2) )
end

--// Plays a sound at the specified position
function bv.sound3(sound, x, y, id)
	parse('sv_soundpos "' .. sound .. '" ' .. x .. ' ' .. y .. ' ' .. (id or ''))
end

--// Adds money to a player
function bv.add_money(id, amount)
	local money = player(id, "money") + amount
	parse('setmoney ' .. id .. ' ' .. money)
end

--// Displays a server message
function bv.sv_msg(txt)
	msg("\169255255255" .. bv.BANK_NAME .. ": " .. txt)
end

--// Returns the number of counter-terrorists in the server
function bv.ct_count()
	local c = 0

	--// Iterates through each player
	for _, id in pairs(player(0, "table")) do
		--// This player is CT
		if player(id, "team") == 2 then
			c = c + 1
		end
	end

	return c
end

--// This function is called whenever a player press "USE"
function bv.use(id, event, data, x, y)
	--// The use event must have no action
	if event ~= 0 then return end

	--// Carrying a suitcase
	if bv.carrying[id] then
		local suitcase = bv.SuitCases[bv.carrying[id]]
		local origin_vault = bv.Vaults[suitcase.from]
		local dist_vault = bv.dist(origin_vault.tx, origin_vault.ty,
			player(id, "tilex"), player(id, "tiley"))
			
		--// Only terrorists can collect the money
		if player(id, "team") == 1 then
			--// This player is far enough from the vault
			if dist_vault >= bv.SUITCASE_COLLECT_RANGE_FROM_VAULT then
				bv.add_money(id, suitcase.money)
				freeimage(suitcase.image_id)
				
				bv.sound3(bv.COLLECT_SOUND, player(id, "x"), player(id, "y"))
				msg2(id, "\169255255255[BankVault]: You have found \169000255000" .. 
				suitcase.money .. bv.BANK_CURRENCY)

				bv.SuitCases[bv.carrying[id]] = nil
				bv.carrying[id] = nil
			else
				msg2(id, "\169255255255[BankVault]: You are still too close to the bank vault !!")
			end
		else
			msg2(id, "\169255255255[BankVault]: Drop this suitcase on the bank vault !!")
		end
		
		return
	end

	--// Only TT can rob
	local player_team = player(id, "team")

	--// For each vault
	for vault_id, vault in pairs(bv.Vaults) do
		--// distance between the player and the vault
		local dist = bv.dist(player(id, "tilex"), player(id, "tiley"), 
			vault.tx, vault.ty)

		--// The player can interacts with the vault
		if dist <= bv.VAULT_USE_RANGE then
			--// The vault is OK
			if vault.status == bv.STATE.SAFE then
				--// Terrorists and can rob
				if player_team == 1 and vault.robbing_timer == 0 and bv.ct_count() >= 1 then
					vault.status = bv.STATE.ROBBED

					bv.sound3(bv.ROBBERY_MUSIC, vault.x, vault.y)
					bv.sv_msg(player(id, "name") .. " is robbing a bank vault !")
					bv.sv_msg( "\169255000000YOU MUST KILL HIM !" )
				else
					--// TIP to terrorists
					if player_team == 1 then
						if vault.robbing_timer > 0 then
							msg2(id, "This bank vault can be robbed in " ..
								vault.robbing_timer .. "s")
						else
							msg2(id, "\169255255255[BankVault]: There must be one CT " ..
								"in order to start a robbery !")
						end
					else
						msg2(id, "\169255255255" .. bv.BANK_NAME .. ": \169000255000" 
						.. vault.money .. bv.BANK_CURRENCY)
					end
				end
			--// The vault is currently robbed
			else
				--// Stops the robbing
				if player_team == 2 then
					bv.reward_player(id)
					bv.reset_vault(vault)
				else
					--// Can collect the money
					if vault.use_timer == 0 then
						vault.use_timer = bv.SUITCASE_USE_DELAY

						--// The vault has still money left
						if vault.money > 0 then
							bv.new_suitcase(vault_id)
						else
							msg2(id, 
								"\169255255255[BankVault]: There are no more money here !")
						end
					else
						msg2(id, "\169255255255[BankVault]: You have to wait \169255000000" ..
							vault.use_timer .. "s")
					end
				end
			end 

			break
		end
	end

	--// For each suitcase
	for suitcase_id, suitcase in pairs(bv.SuitCases) do
		--// The distance between the player and the suitcase
		local dist = bv.dist(player(id, "tilex"), player(id, "tiley"), 
			suitcase.tx, suitcase.ty)

		--// This player can interacts with the suitcase
		if dist <= bv.SUITCASE_USE_RANGE
			and suitcase.status ~= bv.STATE.CARRIED then
			bv.collect_suitcase(id, suitcase_id)
			break
		end
	end
end

--// This function is called whenever a new round starts
function bv.startround(mode)
	--// Refreshing all images ids
	for _, vault in pairs( bv.Vaults ) do
		--// the vault is currently displayed
		if vault.image_id then
			freeimage(vault.image_id)
			vault.image_id = image(bv.VAULT_MODEL, vault.x, vault.y, 0)
		end
	end

	--// Same applies to suitcases
	for _, suitcase in pairs( bv.SuitCases ) do
		--// the wallet is currently displayed
		if suitcase.image_id then
			--// Remove the timer in case of forced restart
			freetimer("freeimage", suitcase.image_id)

			--// Refresh the image
			freeimage(suitcase.image_id)
			suitcase.image_id = image(bv.SUITCASE_DROPPED_MODEL, suitcase.x, suitcase.y, 0)

			--// Update the state
			suitcase.status = bv.STATE.DROPPED
		end
	end

	--// Reset player vars
	for pid, _ in pairs(bv.carrying) do
		bv.carrying[pid] = false
	end
end

--// This function is called whenever a command is parsed
function bv.parse(text) 
	--// Bank Vault spawn command
	if match(text, "!bank_vault%s(%d+)%s(%d+)") then
		local x, y = match(text, "!bank_vault%s(%d+)%s(%d+)")
		x, y = tonumber(x), tonumber(y)

		bv.new_vault(x, y) 
		return 1
	end
end

--// This function is called on player join
function bv.join(id)
	bv.carrying[id] = false
end

--// This function is called on player leave
function bv.leave(id)
	bv.drop_suitcase(id)
end

--// This function is called when a player died
function bv.die(victim)
	bv.drop_suitcase(victim)
end

--// This function is called when a player joins an other team
function bv.team(id)
	bv.drop_suitcase(id)
end

--// This function is called when a player tries to drop an item
function bv.drop(id, iid, type)
	--// Carrying his knife
	if type == 50 then
		bv.drop_suitcase(id)
	end
end

--// Updates bank vaults timers
function bv.second()
	--// For each bank vault
	for vault_id, vault in pairs(bv.Vaults) do
		--// If the bank vault isn't currently robbed
		if vault.status ~= bv.STATE.ROBBED then
			vault.money = vault.money + random(bv.MIN_INCREASE_AMOUNT, 
				bv.MAX_INCREASE_AMOUNT)

			--// Decrease the robbing timer if > 0
			if vault.robbing_timer > 0 then
				vault.robbing_timer = vault.robbing_timer - 1
			end
		else
			-- Still robbing phase
			if vault.robbing_timer < bv.ROBBERY_DURATION then
				vault.robbing_timer = vault.robbing_timer + 1

				-- Decrease the use timer
				if vault.use_timer > 0 then
					vault.use_timer = vault.use_timer - 1
				end
			else
				--// Resetting the bank vault
				bv.reset_vault(vault)
			end
		end
	end

	--// For each suitcase
	for suitcase_id, suitcase in pairs(bv.SuitCases) do
		--// The suitcase is currently on the ground
		if suitcase.status == bv.STATE.DROPPED then
			suitcase.disappear_timer = suitcase.disappear_timer + 1

			--// This suitcase must be removed
			if suitcase.disappear_timer == bv.SUITCASE_DISAPPEAR_DELAY then
				tween_alpha(suitcase.image_id, 1000, 0)
				timer(1000, "freeimage", suitcase.image_id)
				bv.SuitCases[suitcase_id] = nil
			end
		end
	end
end

--// Creates a new bank vault at the specified position
function bv.new_vault(x, y)
	bv.Vaults[#bv.Vaults + 1] = {
		x = x,
		y = y,
		tx = floor(x / 32),
		ty = floor(y / 32),
		money = bv.START_MONEY,
		use_timer = 0,
		robbing_timer = 0,
		status = bv.STATE.SAFE,
		image_id = image(bv.VAULT_MODEL, x, y, 0),
	}
end

--// Creates a new suitcase from a bank vault
function bv.new_suitcase(vault_id)
	local vault = bv.Vaults[vault_id]

	bv.SuitCases[#bv.SuitCases + 1] = {
		from = vault_id,
		x = vault.x,
		y = vault.y,
		tx = vault.tx,
		ty = vault.ty,
		disappear_timer = 0,
		status = bv.STATE.DROPPED,
		money = bv.get_money_from(vault),
		image_id = image(bv.SUITCASE_DROPPED_MODEL, vault.x, vault.y, 0)
	}
end

--// Returns a random amount of money from this vault
function bv.get_money_from(vault)
	--// It's still possible to get money from this vault
	if vault.money > 0 then
		--// Random amount
		local amount = random(bv.MIN_MONEY_AMOUNT, bv.MAX_MONEY_AMOUNT)

		--// Fix money
		if amount > vault.money then
			amount = vault.money
		end

		--// Update the vault
		vault.money = vault.money - amount

		return amount
	end

	return 0
end

--// Drops the suitcase at the player position
function bv.drop_suitcase(id)
	local suitcase_id = bv.carrying[id]

	--// This player is carrying a suitcase
	if suitcase_id then
		local suitcase = bv.SuitCases[suitcase_id]

		--// Removes the old image
		freeimage(suitcase.image_id)

		--// Drop the suitcase at the current player position
		local x, y = player(id, "x"), player(id, "y")
		suitcase.image_id = image(bv.SUITCASE_DROPPED_MODEL, x, y, 0)
		imagepos(suitcase.image_id, x, y, player(id, "rot"))

		--// Update state
		suitcase.disappear_timer = 0
		suitcase.status = bv.STATE.DROPPED
		suitcase.x = x
		suitcase.y = y
		suitcase.tx = player(id, "tilex")
		suitcase.ty = player(id, "tiley")

		--// Dropped by a CT
		if player(id, "team") == 2 then
			--// Through all bank vaults
			for _, vault in pairs(bv.Vaults) do
				--// Looking for the closest one
				local dist = bv.dist(suitcase.tx, suitcase.ty, 
					vault.tx, vault.ty)

				--// The bank vault can collect this suitcase !
				if dist <= bv.VAULT_USE_RANGE then
					vault.money = vault.money + suitcase.money
					bv.sv_msg("Thanks to " .. player(id, "name") .. 
						" who has saved \169000255000" .. suitcase.money .. bv.BANK_CURRENCY)

					--// Remove the suitcase
					freeimage(suitcase.image_id)
					bv.SuitCases[suitcase_id] = nil
					break
				end
			end
		end
	end

	bv.carrying[id] = nil
end

--// Collects a suitcase
function bv.collect_suitcase(id, suitcase_id)
	local suitcase = bv.SuitCases[suitcase_id]

	--// The suitcase must be displayed
	if suitcase.image_id then
		--// Replace the old image
		freeimage(suitcase.image_id)
		suitcase.image_id = image(bv.SUITCASE_MODEL, 1, 0, 200 + id)
		bv.carrying[id] = suitcase_id

		--// Update state
		suitcase.status = bv.STATE.CARRIED
	end
end

--// Resets a bank vault
function bv.reset_vault(vault)
	vault.status = bv.STATE.SAFE
	vault.use_timer = 0
	vault.robbing_timer = random(bv.MIN_ROBBING_INTERVAL, 
		bv.MAX_ROBBING_INTERVAL)
end

--// Rewards a ct for saving a bank vault
function bv.reward_player(id)
	local reward = random(bv.MIN_REWARD, bv.MAX_REWARD)
	
	bv.add_money(id, reward)
	msg2(id, "\169255255255" .. bv.BANK_NAME .. 
		' gave to you \169000255000' .. reward .. bv.BANK_CURRENCY)
	
	bv.sv_msg(player(id, "name") .. " has saved the bank vault !")
end          

--// Attaching hooks functions
addhook("join", "bv.join")
addhook("leave", "bv.leave")
addhook("use", "bv.use")
addhook("startround", "bv.startround")
addhook("parse", "bv.parse")
addhook("second", "bv.second")
addhook("team", "bv.team")
addhook("die", "bv.die")
addhook("drop", "bv.drop")